"""Quarkupy Implementation for ingesting files from various sources, extracting text, chunking it, and preparing
curated context for Agents and other AI applications."""
import logging
import os
import pprint
import datetime
from typing import Union, Any
from urllib.parse import urlparse

import uuid_utils as uuid  # Replace standard uuid with uuid_utils for UUID7
from halo import Halo

import quarkupy
import quarkupy.lib.runner as qr
from quarkupy.types.history import FlowHistoryItem


API_KEY = os.environ.get("QUARK_API_KEY")
API_END_POINT = "https://demo.quarklabs.ai/api/quark"
OPENAI_API_KEY = os.environ.get("OPENAI_API_KEY")
AWS_ACCESS_KEY_ID = os.environ.get("AWS_ACCESS_KEY_ID")
AWS_ACCESS_KEY_SECRET = os.environ.get("AWS_ACCESS_KEY_SECRET")
AWS_ENDPOINT = os.environ.get("AWS_ENDPOINT")
S3_BUCKET = os.environ.get("S3_BUCKET")
VECTOR_DB_TABLE = os.environ.get("VECTOR_DB_TABLE")

PRINT_DATASETS = False
PRINT_QUARK_METRICS = False
PRINT_QUARK_HISTORY = False
SPINNER = "dots"

host_url = urlparse(API_END_POINT)
url_root = f"{host_url.scheme}://{host_url.netloc}"


async def util_save_flow_history(history: FlowHistoryItem) -> None:
    """
    Saves the Quark history (status) on the Quark services via a REST call. This call is an "upsert" call, meaning
    if a history item with the same `quark_id` exists, it will be updated with the new status. If not, a new history
    item will be created.

    :raises Exception: If there is an error during the API operation.
    """
    api = quarkupy.AsyncClient(api_key=API_KEY, base_url=API_END_POINT)
    await api.history.flow.update(**history.model_dump())
    await api.close()


printer = pprint.PrettyPrinter(indent=2)


async def run_quark(q, flow_history: FlowHistoryItem) -> Union[uuid.UUID, None]:
    """Common control flow for running quarks"""
    try:
        await q.start()
        await q.execute()
        print(f"💠 Quark ID: {q.quark_id} - more: {url_root}/q/history/quark/{q.quark_id}")

        # Runs are asynchronous, so we need to poll until the run is complete
        done = await q.poll_until_complete(timeout_min=360)

        flow_history.nodes.append(q.history.quark_history_id)

        if PRINT_QUARK_HISTORY:
            printer.pprint(done.model_dump())

        if PRINT_QUARK_METRICS:
            printer.pprint(done.state)

        try:
            print(
                f"💠 Got dataset with ID: {done.output['dataset_uuid']} - preview: {url_root}/q/data-manager/quark-outputs/{done.output['dataset_uuid']}"
            )
            if PRINT_DATASETS:
                t = await q.read_output_dataset()
                if t is not None:
                    printer.pprint(t.to_pandas())
                else:
                    print("No dataset produced")

            output_dataset_uuid = done.output["dataset_uuid"]
            if output_dataset_uuid is None:
                logging.error("Output dataset UUID is None")
                raise Exception("Output dataset UUID is None")

            return output_dataset_uuid
        except KeyError:
            return None

    except Exception as e:
        flow_history.status = "Failed"
        await util_save_flow_history(flow_history)

        printer.pprint(e)
        raise e


async def step1_make_classifier_prompt(flow: FlowHistoryItem, dataset_uuid: uuid.UUID, classifier_id: str) -> uuid.UUID:
    with Halo(text="💠 Creating classifier prompts...\n", spinner=SPINNER):
        q_input = qr.ContextClassifierPromptInput(
            flow_id=flow.flow_history_id,
            ipc_dataset_id=dataset_uuid.__str__(),
            opt_rendered_col="prompt",
            classifier_ids=[classifier_id]
        )

        q = qr.ClassifierPromptQuark(
            lattice_id=flow.flow_history_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q, flow)

async def step2_classifier_inference(flow: FlowHistoryItem, dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Running Classification Inference...\n", spinner=SPINNER):
        q_input = qr.OpenAICompletionsInput(
            lattice_id=flow.flow_history_id,
            ipc_dataset_id=dataset_uuid.__str__(),
            api_key=OPENAI_API_KEY,
            opt_model_name="gpt-4.1-nano",
            opt_prompt_column="prompt",
            opt_json_output=True,
            opt_system_prompt="""
You will be assigned tasks involving the extraction, analysis, and interpretation of data within various types of documents. Your role is to carefully process the information and provide accurate, relevant outputs based on the specific instructions provided.

KEY TERMINOLOGY TO UNDERSTAND:

1. Document: A structured or unstructured file containing written or visual content. Documents can vary in type and purpose, including but not limited to:
- Contracts: Legal agreements defining terms and obligations.
- Invoices: Financial documents detailing transactions and payments.
- Curricula Vitae (CVs): Resumes outlining an individual's professional experience and qualifications.
- General Documents: Any other types of files that may contain text, tables, images, or mixed formats.
Note that occasionally you may be working with document fragments rather than complete documents. These fragments represent portions of a larger document and should be analyzed within their limited context. When working with fragments, focus on extracting information from the available content without making assumptions about missing parts.

2. Aspect: A defined area or topic within a document that requires focused attention. Each aspect corresponds to a specific subject or theme described in the task. For example:
- Contract Aspects: Payment terms, parties involved, or termination clauses.
- Invoice Aspects: Due dates, line-item breakdowns, or tax details.
- CV Aspects: Work experience, education, or skills.
You will analyze aspects as instructed, considering their relevance and context within the document.

3. Concept: A unit of information or an entity relevant to the task. Concepts may be derived from an aspect or the broader document context. They represent a wide range of data points and insights, from simple entities (names, dates, monetary values) to complex evaluations, conclusions, and answers to specific questions. Concepts can be:
- Factual extractions: Such as a penalty clause in a contract, a total amount due in an invoice, or a certification in a CV.
- Analytical insights: Such as risk assessments, compliance evaluations, or pattern identifications.
- Reasoned conclusions: Such as determining whether a document meets specific criteria or answers particular questions.
- Interpretative judgments: Such as ratings, classifications, or qualitative assessments based on document content.

GUIDELINES FOR YOUR WORK:
- Understand the context: Before processing the document, ensure you comprehend the nature of the document and the aspects or concepts you need to focus on.
- Follow specific instructions: Each task will include detailed instructions, such as which aspects or concepts to focus on and how the data should be presented.
- Maintain accuracy: Provide precise extractions and analysis, ensuring your outputs align with the document's intent and context.
- Adapt to variability: Documents may differ in structure, language, or formatting. Be flexible and apply reasoning to handle variations effectively.

EXPECTED DELIVERABLES:
- Structured outputs: Clearly formatted and well-organized results based on the task requirements.
- Explanations (when required): When required by the instructions, include justifications or reasoning for your interpretations.
- Insights (when required): When required by the instructions, highlight trends, patterns, or noteworthy findings that could add value to the task.
- References (when required): When required by the instructions, output references based on which you extracted data, provided insights, or made conclusions during the task.

By adhering to this framework, you will ensure consistent and high-quality performance across diverse document analysis tasks.
""",
        )

        q = qr.OpenAICompletionBaseQuark(
            lattice_id=flow.flow_history_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q, flow)

async def step3_parse_classifier_llm(flow: FlowHistoryItem, dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Parsing LLM Responses for Classifiers...\n", spinner=SPINNER):
        q_input = qr.ParseClassifierLlmInput(
            flow_id=flow.flow_history_id,
            ipc_dataset_id=dataset_uuid.__str__()
        )

        q = qr.ClassifierParserQuark(
            lattice_id=flow.flow_history_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q, flow)

async def step4_store_classified_segments(flow: FlowHistoryItem, dataset_uuid: uuid.UUID) -> Any:
    with Halo(text="💠 Storing segment classifications in Quark ContextStore\n", spinner=SPINNER):
        q_input = qr.ContextInsertClassifiedSegmentsInput(
            flow_id=flow.flow_history_id,
            ipc_dataset_id=dataset_uuid.__str__()
        )

        q = qr.ContextInsertClassifiedSegmentsQuark(
            lattice_id=flow.flow_history_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q, flow)

async def _classify_segments(classifier_id: str, dataset_uuid: uuid.UUID) -> Any:
    # Mock lattice id for grouping Quarks
    flow_id = uuid.uuid7().__str__()
    flow_identifier = "lattice:classify_segments"

    flow = qr.FlowHistoryItem(
        flow_history_id=flow_id,
        registry_qrn=flow_identifier,
        created_at=datetime.datetime.now(datetime.timezone.utc),
        updated_at=datetime.datetime.now(datetime.timezone.utc),
        nodes=[],
        edges=[],
        input=None,
        output=None,
        metrics=None,
        status="New",
        identity_id="019738eb-eb84-77d1-b0be-1514c0a9ed6a",
    )

    flow.status = "Running"
    await util_save_flow_history(flow)

    try:
        await step4_store_classified_segments(flow,
            await step3_parse_classifier_llm(flow,
                await step2_classifier_inference(flow,
                    await step1_make_classifier_prompt(flow, dataset_uuid, classifier_id)
                )
            )
        )

        # Finalize the lattice history
        flow.status = "Completed"
        await util_save_flow_history(flow)

        print("Done! 🎉")
    except KeyboardInterrupt:
        flow.status = "Failed"
        await util_save_flow_history(flow)
        print("Interrupted")
    except Exception as e:
        flow.status = "Failed"
        await util_save_flow_history(flow)
        raise e

def classify_segments(classifier_id: str, dataset_uuid: uuid.UUID) -> Any:
    import asyncio

    asyncio.run(_classify_segments(classifier_id, dataset_uuid))


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .role import (
    RoleResource,
    AsyncRoleResource,
    RoleResourceWithRawResponse,
    AsyncRoleResourceWithRawResponse,
    RoleResourceWithStreamingResponse,
    AsyncRoleResourceWithStreamingResponse,
)
from .identity import (
    IdentityResource,
    AsyncIdentityResource,
    IdentityResourceWithRawResponse,
    AsyncIdentityResourceWithRawResponse,
    IdentityResourceWithStreamingResponse,
    AsyncIdentityResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource

__all__ = ["MembersResource", "AsyncMembersResource"]


class MembersResource(SyncAPIResource):
    @cached_property
    def identity(self) -> IdentityResource:
        return IdentityResource(self._client)

    @cached_property
    def role(self) -> RoleResource:
        return RoleResource(self._client)

    @cached_property
    def with_raw_response(self) -> MembersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return MembersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MembersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return MembersResourceWithStreamingResponse(self)


class AsyncMembersResource(AsyncAPIResource):
    @cached_property
    def identity(self) -> AsyncIdentityResource:
        return AsyncIdentityResource(self._client)

    @cached_property
    def role(self) -> AsyncRoleResource:
        return AsyncRoleResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMembersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncMembersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMembersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncMembersResourceWithStreamingResponse(self)


class MembersResourceWithRawResponse:
    def __init__(self, members: MembersResource) -> None:
        self._members = members

    @cached_property
    def identity(self) -> IdentityResourceWithRawResponse:
        return IdentityResourceWithRawResponse(self._members.identity)

    @cached_property
    def role(self) -> RoleResourceWithRawResponse:
        return RoleResourceWithRawResponse(self._members.role)


class AsyncMembersResourceWithRawResponse:
    def __init__(self, members: AsyncMembersResource) -> None:
        self._members = members

    @cached_property
    def identity(self) -> AsyncIdentityResourceWithRawResponse:
        return AsyncIdentityResourceWithRawResponse(self._members.identity)

    @cached_property
    def role(self) -> AsyncRoleResourceWithRawResponse:
        return AsyncRoleResourceWithRawResponse(self._members.role)


class MembersResourceWithStreamingResponse:
    def __init__(self, members: MembersResource) -> None:
        self._members = members

    @cached_property
    def identity(self) -> IdentityResourceWithStreamingResponse:
        return IdentityResourceWithStreamingResponse(self._members.identity)

    @cached_property
    def role(self) -> RoleResourceWithStreamingResponse:
        return RoleResourceWithStreamingResponse(self._members.role)


class AsyncMembersResourceWithStreamingResponse:
    def __init__(self, members: AsyncMembersResource) -> None:
        self._members = members

    @cached_property
    def identity(self) -> AsyncIdentityResourceWithStreamingResponse:
        return AsyncIdentityResourceWithStreamingResponse(self._members.identity)

    @cached_property
    def role(self) -> AsyncRoleResourceWithStreamingResponse:
        return AsyncRoleResourceWithStreamingResponse(self._members.role)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .opendal import (
    OpendalResource,
    AsyncOpendalResource,
    OpendalResourceWithRawResponse,
    AsyncOpendalResourceWithRawResponse,
    OpendalResourceWithStreamingResponse,
    AsyncOpendalResourceWithStreamingResponse,
)
from .s3_read_csv import (
    S3ReadCsvResource,
    AsyncS3ReadCsvResource,
    S3ReadCsvResourceWithRawResponse,
    AsyncS3ReadCsvResourceWithRawResponse,
    S3ReadCsvResourceWithStreamingResponse,
    AsyncS3ReadCsvResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from .s3_read_files_binary import (
    S3ReadFilesBinaryResource,
    AsyncS3ReadFilesBinaryResource,
    S3ReadFilesBinaryResourceWithRawResponse,
    AsyncS3ReadFilesBinaryResourceWithRawResponse,
    S3ReadFilesBinaryResourceWithStreamingResponse,
    AsyncS3ReadFilesBinaryResourceWithStreamingResponse,
)

__all__ = ["FilesResource", "AsyncFilesResource"]


class FilesResource(SyncAPIResource):
    @cached_property
    def s3_read_files_binary(self) -> S3ReadFilesBinaryResource:
        return S3ReadFilesBinaryResource(self._client)

    @cached_property
    def s3_read_csv(self) -> S3ReadCsvResource:
        return S3ReadCsvResource(self._client)

    @cached_property
    def opendal(self) -> OpendalResource:
        return OpendalResource(self._client)

    @cached_property
    def with_raw_response(self) -> FilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return FilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return FilesResourceWithStreamingResponse(self)


class AsyncFilesResource(AsyncAPIResource):
    @cached_property
    def s3_read_files_binary(self) -> AsyncS3ReadFilesBinaryResource:
        return AsyncS3ReadFilesBinaryResource(self._client)

    @cached_property
    def s3_read_csv(self) -> AsyncS3ReadCsvResource:
        return AsyncS3ReadCsvResource(self._client)

    @cached_property
    def opendal(self) -> AsyncOpendalResource:
        return AsyncOpendalResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncFilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncFilesResourceWithStreamingResponse(self)


class FilesResourceWithRawResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

    @cached_property
    def s3_read_files_binary(self) -> S3ReadFilesBinaryResourceWithRawResponse:
        return S3ReadFilesBinaryResourceWithRawResponse(self._files.s3_read_files_binary)

    @cached_property
    def s3_read_csv(self) -> S3ReadCsvResourceWithRawResponse:
        return S3ReadCsvResourceWithRawResponse(self._files.s3_read_csv)

    @cached_property
    def opendal(self) -> OpendalResourceWithRawResponse:
        return OpendalResourceWithRawResponse(self._files.opendal)


class AsyncFilesResourceWithRawResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

    @cached_property
    def s3_read_files_binary(self) -> AsyncS3ReadFilesBinaryResourceWithRawResponse:
        return AsyncS3ReadFilesBinaryResourceWithRawResponse(self._files.s3_read_files_binary)

    @cached_property
    def s3_read_csv(self) -> AsyncS3ReadCsvResourceWithRawResponse:
        return AsyncS3ReadCsvResourceWithRawResponse(self._files.s3_read_csv)

    @cached_property
    def opendal(self) -> AsyncOpendalResourceWithRawResponse:
        return AsyncOpendalResourceWithRawResponse(self._files.opendal)


class FilesResourceWithStreamingResponse:
    def __init__(self, files: FilesResource) -> None:
        self._files = files

    @cached_property
    def s3_read_files_binary(self) -> S3ReadFilesBinaryResourceWithStreamingResponse:
        return S3ReadFilesBinaryResourceWithStreamingResponse(self._files.s3_read_files_binary)

    @cached_property
    def s3_read_csv(self) -> S3ReadCsvResourceWithStreamingResponse:
        return S3ReadCsvResourceWithStreamingResponse(self._files.s3_read_csv)

    @cached_property
    def opendal(self) -> OpendalResourceWithStreamingResponse:
        return OpendalResourceWithStreamingResponse(self._files.opendal)


class AsyncFilesResourceWithStreamingResponse:
    def __init__(self, files: AsyncFilesResource) -> None:
        self._files = files

    @cached_property
    def s3_read_files_binary(self) -> AsyncS3ReadFilesBinaryResourceWithStreamingResponse:
        return AsyncS3ReadFilesBinaryResourceWithStreamingResponse(self._files.s3_read_files_binary)

    @cached_property
    def s3_read_csv(self) -> AsyncS3ReadCsvResourceWithStreamingResponse:
        return AsyncS3ReadCsvResourceWithStreamingResponse(self._files.s3_read_csv)

    @cached_property
    def opendal(self) -> AsyncOpendalResourceWithStreamingResponse:
        return AsyncOpendalResourceWithStreamingResponse(self._files.opendal)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from quarkupy import Quark, AsyncQuark
from tests.utils import assert_matches_type
from quarkupy.types import (
    DataSetInfo,
    DatasetListResponse,
)
from quarkupy._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDataset:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Quark) -> None:
        dataset = client.dataset.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DataSetInfo, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Quark) -> None:
        response = client.dataset.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DataSetInfo, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Quark) -> None:
        with client.dataset.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DataSetInfo, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Quark) -> None:
        dataset = client.dataset.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Quark) -> None:
        response = client.dataset.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Quark) -> None:
        with client.dataset.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_arrow(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = client.dataset.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_arrow_with_all_params(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = client.dataset.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_arrow(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        dataset = client.dataset.with_raw_response.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_arrow(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.dataset.with_streaming_response.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, StreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_arrow(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve_arrow(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_chunks(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        dataset = client.dataset.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_chunks_with_all_params(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        dataset = client.dataset.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_chunks(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        dataset = client.dataset.with_raw_response.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_chunks(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.dataset.with_streaming_response.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, StreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_chunks(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve_chunks(
                file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.dataset.with_raw_response.retrieve_chunks(
                file_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_csv(self, client: Quark) -> None:
        dataset = client.dataset.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_csv_with_all_params(self, client: Quark) -> None:
        dataset = client.dataset.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=0,
            offset=0,
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_csv(self, client: Quark) -> None:
        response = client.dataset.with_raw_response.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_csv(self, client: Quark) -> None:
        with client.dataset.with_streaming_response.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(str, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve_csv(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve_csv(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_files(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = client.dataset.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_files_with_all_params(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = client.dataset.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_files(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        dataset = client.dataset.with_raw_response.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_files(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.dataset.with_streaming_response.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, StreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_files(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve_files(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_json(self, client: Quark) -> None:
        dataset = client.dataset.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_json_with_all_params(self, client: Quark) -> None:
        dataset = client.dataset.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            max_cell_size=0,
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_json(self, client: Quark) -> None:
        response = client.dataset.with_raw_response.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = response.parse()
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_json(self, client: Quark) -> None:
        with client.dataset.with_streaming_response.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = response.parse()
            assert_matches_type(str, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve_json(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.dataset.with_raw_response.retrieve_json(
                id="",
            )


class TestAsyncDataset:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DataSetInfo, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncQuark) -> None:
        response = await async_client.dataset.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DataSetInfo, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncQuark) -> None:
        async with async_client.dataset.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DataSetInfo, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.list()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncQuark) -> None:
        response = await async_client.dataset.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(DatasetListResponse, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncQuark) -> None:
        async with async_client.dataset.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(DatasetListResponse, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_arrow(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = await async_client.dataset.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_arrow_with_all_params(
        self, async_client: AsyncQuark, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = await async_client.dataset.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_arrow(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        dataset = await async_client.dataset.with_raw_response.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_arrow(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/arrow").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.dataset.with_streaming_response.retrieve_arrow(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_arrow(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_arrow(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_chunks(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        dataset = await async_client.dataset.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_chunks_with_all_params(
        self, async_client: AsyncQuark, respx_mock: MockRouter
    ) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        dataset = await async_client.dataset.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_chunks(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        dataset = await async_client.dataset.with_raw_response.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_chunks(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get(
            "/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/chunks"
        ).mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.dataset.with_streaming_response.retrieve_chunks(
            file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_chunks(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_chunks(
                file_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_chunks(
                file_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_csv(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_csv_with_all_params(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            limit=0,
            offset=0,
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_csv(self, async_client: AsyncQuark) -> None:
        response = await async_client.dataset.with_raw_response.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_csv(self, async_client: AsyncQuark) -> None:
        async with async_client.dataset.with_streaming_response.retrieve_csv(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(str, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve_csv(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_csv(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_files(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = await async_client.dataset.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_files_with_all_params(
        self, async_client: AsyncQuark, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        dataset = await async_client.dataset.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            _limit=0,
            _offset=0,
        )
        assert dataset.is_closed
        assert await dataset.json() == {"foo": "bar"}
        assert cast(Any, dataset.is_closed) is True
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_files(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        dataset = await async_client.dataset.with_raw_response.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert dataset.is_closed is True
        assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await dataset.json() == {"foo": "bar"}
        assert isinstance(dataset, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_files(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/dataset/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/files").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.dataset.with_streaming_response.retrieve_files(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as dataset:
            assert not dataset.is_closed
            assert dataset.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await dataset.json() == {"foo": "bar"}
            assert cast(Any, dataset.is_closed) is True
            assert isinstance(dataset, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, dataset.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_files(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_files(
                id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_json(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_json_with_all_params(self, async_client: AsyncQuark) -> None:
        dataset = await async_client.dataset.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            max_cell_size=0,
        )
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_json(self, async_client: AsyncQuark) -> None:
        response = await async_client.dataset.with_raw_response.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset = await response.parse()
        assert_matches_type(str, dataset, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_json(self, async_client: AsyncQuark) -> None:
        async with async_client.dataset.with_streaming_response.retrieve_json(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset = await response.parse()
            assert_matches_type(str, dataset, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve_json(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.dataset.with_raw_response.retrieve_json(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from quarkupy import Quark, AsyncQuark
from tests.utils import assert_matches_type
from quarkupy.types.worker import (
    ManagementRetrieveResponse,
    ManagementRetrieveTokioResponse,
    ManagementRetrievePythonStatusResponse,
)
from quarkupy.types.context import SuccessResponseMessage
from quarkupy.types.admin.identity import IdentityModel

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestManagement:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Quark) -> None:
        management = client.worker.management.retrieve()
        assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Quark) -> None:
        response = client.worker.management.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = response.parse()
        assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Quark) -> None:
        with client.worker.management.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = response.parse()
            assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_ping(self, client: Quark) -> None:
        management = client.worker.management.ping()
        assert_matches_type(SuccessResponseMessage, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_ping(self, client: Quark) -> None:
        response = client.worker.management.with_raw_response.ping()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = response.parse()
        assert_matches_type(SuccessResponseMessage, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_ping(self, client: Quark) -> None:
        with client.worker.management.with_streaming_response.ping() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = response.parse()
            assert_matches_type(SuccessResponseMessage, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_auth_status(self, client: Quark) -> None:
        management = client.worker.management.retrieve_auth_status()
        assert_matches_type(IdentityModel, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_auth_status(self, client: Quark) -> None:
        response = client.worker.management.with_raw_response.retrieve_auth_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = response.parse()
        assert_matches_type(IdentityModel, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_auth_status(self, client: Quark) -> None:
        with client.worker.management.with_streaming_response.retrieve_auth_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = response.parse()
            assert_matches_type(IdentityModel, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_python_status(self, client: Quark) -> None:
        management = client.worker.management.retrieve_python_status()
        assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_python_status(self, client: Quark) -> None:
        response = client.worker.management.with_raw_response.retrieve_python_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = response.parse()
        assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_python_status(self, client: Quark) -> None:
        with client.worker.management.with_streaming_response.retrieve_python_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = response.parse()
            assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_tokio(self, client: Quark) -> None:
        management = client.worker.management.retrieve_tokio()
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_tokio_with_all_params(self, client: Quark) -> None:
        management = client.worker.management.retrieve_tokio(
            with_dump=True,
        )
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_tokio(self, client: Quark) -> None:
        response = client.worker.management.with_raw_response.retrieve_tokio()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = response.parse()
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_tokio(self, client: Quark) -> None:
        with client.worker.management.with_streaming_response.retrieve_tokio() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = response.parse()
            assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncManagement:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.retrieve()
        assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncQuark) -> None:
        response = await async_client.worker.management.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = await response.parse()
        assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncQuark) -> None:
        async with async_client.worker.management.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = await response.parse()
            assert_matches_type(ManagementRetrieveResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_ping(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.ping()
        assert_matches_type(SuccessResponseMessage, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_ping(self, async_client: AsyncQuark) -> None:
        response = await async_client.worker.management.with_raw_response.ping()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = await response.parse()
        assert_matches_type(SuccessResponseMessage, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_ping(self, async_client: AsyncQuark) -> None:
        async with async_client.worker.management.with_streaming_response.ping() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = await response.parse()
            assert_matches_type(SuccessResponseMessage, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_auth_status(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.retrieve_auth_status()
        assert_matches_type(IdentityModel, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_auth_status(self, async_client: AsyncQuark) -> None:
        response = await async_client.worker.management.with_raw_response.retrieve_auth_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = await response.parse()
        assert_matches_type(IdentityModel, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_auth_status(self, async_client: AsyncQuark) -> None:
        async with async_client.worker.management.with_streaming_response.retrieve_auth_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = await response.parse()
            assert_matches_type(IdentityModel, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_python_status(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.retrieve_python_status()
        assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_python_status(self, async_client: AsyncQuark) -> None:
        response = await async_client.worker.management.with_raw_response.retrieve_python_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = await response.parse()
        assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_python_status(self, async_client: AsyncQuark) -> None:
        async with async_client.worker.management.with_streaming_response.retrieve_python_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = await response.parse()
            assert_matches_type(ManagementRetrievePythonStatusResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_tokio(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.retrieve_tokio()
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_tokio_with_all_params(self, async_client: AsyncQuark) -> None:
        management = await async_client.worker.management.retrieve_tokio(
            with_dump=True,
        )
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_tokio(self, async_client: AsyncQuark) -> None:
        response = await async_client.worker.management.with_raw_response.retrieve_tokio()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        management = await response.parse()
        assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_tokio(self, async_client: AsyncQuark) -> None:
        async with async_client.worker.management.with_streaming_response.retrieve_tokio() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            management = await response.parse()
            assert_matches_type(ManagementRetrieveTokioResponse, management, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["OpenAIEmbeddingRunParams"]


class OpenAIEmbeddingRunParams(TypedDict, total=False):
    api_key: Required[str]

    ipc_dataset_id: Required[str]

    lattice_id: Required[str]

    opt_model_name: str

    opt_num_embeddings: int

    opt_text_additional_embed_columns: List[str]

    opt_text_column_name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["ManagementRetrieveTokioResponse", "Worker"]


class Worker(BaseModel):
    worker_id: int

    worker_local_schedule_count: int
    """
    Returns the number of tasks scheduled from within the runtime on the given
    worker’s local queue.

    The local schedule count starts at zero when the runtime is created and
    increases by one each time a task is woken from inside of the runtime on the
    given worker. This usually means that a task is spawned or notified from within
    a runtime thread and will be queued on the worker-local queue.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_noop_count: int
    """
    Returns the number of times the given worker thread unparked but performed no
    work before parking again.

    The worker no-op count starts at zero when the runtime is created and increases
    by one each time the worker unparks the thread but finds no new work and goes
    back to sleep. This indicates a false-positive wake up.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_overflow_count: int
    """Returns the number of times the given worker thread saturated its local queue.

    This metric only applies to the multi-threaded scheduler.

    The worker overflow count starts at zero when the runtime is created and
    increases by one each time the worker attempts to schedule a task locally, but
    its local queue is full. When this happens, half of the local queue is moved to
    the injection queue.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_park_count: int
    """Returns the total number of times the given worker thread has parked.

    The worker park count starts at zero when the runtime is created and increases
    by one each time the worker parks the thread waiting for new inbound events to
    process. This usually means the worker has processed all pending work and is
    currently idle.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_park_unpark_count: int
    """
    Returns the total number of times the given worker thread has parked and
    unparked.

    The worker park/unpark count starts at zero when the runtime is created and
    increases by one each time the worker parks the thread waiting for new inbound
    events to process. This usually means the worker has processed all pending work
    and is currently idle. When new work becomes available, the worker is unparked
    and the park/unpark count is again increased by one.

    An odd count means that the worker is currently parked. An even count means that
    the worker is currently active.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_poll_count: int
    """Returns the number of tasks the given worker thread has polled.

    The worker poll count starts at zero when the runtime is created and increases
    by one each time the worker polls a scheduled task.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_steal_count: int
    """
    Returns the number of tasks the given worker thread stole from another worker
    thread.

    This metric only applies to the multi-threaded runtime and will always return 0
    when using the current thread runtime.

    The worker steal count starts at zero when the runtime is created and increases
    by N each time the worker has processed its scheduled queue and successfully
    steals N more pending tasks from another worker.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    worker_steal_operations: int
    """
    Returns the number of times the given worker thread stole tasks from another
    worker thread.

    This metric only applies to the multi-threaded runtime and will always return 0
    when using the current thread runtime.

    The worker steal count starts at zero when the runtime is created and increases
    by one each time the worker has processed its scheduled queue and successfully
    steals more pending tasks from another worker.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """


class ManagementRetrieveTokioResponse(BaseModel):
    budget_forced_yield_count: int
    """
    Returns the number of times that tasks have been forced to yield back to the
    scheduler after exhausting their task budgets.

    This count starts at zero when the runtime is created and increases by one each
    time a task yields due to exhausting its budget.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    dump: List[str]

    global_queue_depth: int
    """Returns the number of tasks currently scheduled in the runtime’s global queue.

    Tasks that are spawned or notified from a non-runtime thread are scheduled using
    the runtime’s global queue. This metric returns the current number of tasks
    pending in the global queue. As such, the returned value may increase or
    decrease as new tasks are scheduled and processed.
    """

    io_driver_fd_deregistered_count: int
    """
    Returns the number of file descriptors that have been deregistered by the
    runtime’s I/O driver.
    """

    io_driver_fd_registered_count: int
    """
    Returns the number of file descriptors that have been registered with the
    runtime’s I/O driver.
    """

    io_driver_ready_count: int
    """Returns the number of ready events processed by the runtime’s I/O driver."""

    num_alive_tasks: int
    """Returns the current number of alive tasks in the runtime.

    This counter increases when a task is spawned and decreases when a task exits.
    """

    num_blocking_threads: int
    """Returns the number of additional threads spawned by the runtime.

    The number of workers is set by configuring max_blocking_threads on
    runtime::Builder.
    """

    num_workers: int
    """Returns the number of worker threads used by the runtime.

    The number of workers is set by configuring worker_threads on runtime::Builder.
    When using the current_thread runtime, the return value is always 1.
    """

    remote_schedule_count: int
    """Returns the number of tasks scheduled from outside of the runtime.

    The remote schedule count starts at zero when the runtime is created and
    increases by one each time a task is woken from outside of the runtime. This
    usually means that a task is spawned or notified from a non-runtime thread and
    must be queued using the Runtime’s injection queue, which tends to be slower.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    spawned_tasks_count: int
    """Returns the number of tasks spawned in this runtime since it was created.

    This count starts at zero when the runtime is created and increases by one each
    time a task is spawned.

    The counter is monotonically increasing. It is never decremented or reset to
    zero.
    """

    workers: List[Worker]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from quarkupy import Quark, AsyncQuark
from tests.utils import assert_matches_type
from quarkupy._utils import parse_datetime
from quarkupy.types.models import MlModelRole, RoleListResponse
from quarkupy.types.context import SuccessResponseMessage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRoles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Quark) -> None:
        role = client.models.roles.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.models.roles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Quark) -> None:
        role = client.models.roles.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Quark) -> None:
        role = client.models.roles.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            updated_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Quark) -> None:
        role = client.models.roles.list()
        assert_matches_type(RoleListResponse, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(RoleListResponse, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(RoleListResponse, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Quark) -> None:
        role = client.models.roles.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SuccessResponseMessage, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(SuccessResponseMessage, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(SuccessResponseMessage, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.models.roles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_schema(self, client: Quark) -> None:
        role = client.models.roles.retrieve_schema()
        assert_matches_type(object, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_schema(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.retrieve_schema()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(object, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_schema(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.retrieve_schema() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(object, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update_partial(self, client: Quark) -> None:
        role = client.models.roles.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_partial_with_all_params(self, client: Quark) -> None:
        role = client.models.roles.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            updated_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_partial(self, client: Quark) -> None:
        response = client.models.roles.with_raw_response.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_partial(self, client: Quark) -> None:
        with client.models.roles.with_streaming_response.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_partial(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.models.roles.with_raw_response.update_partial(
                id="",
                config_override={},
                model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                name="name",
                owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncRoles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.models.roles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            updated_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.update(
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.list()
        assert_matches_type(RoleListResponse, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(RoleListResponse, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(RoleListResponse, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(SuccessResponseMessage, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(SuccessResponseMessage, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(SuccessResponseMessage, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.models.roles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_schema(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.retrieve_schema()
        assert_matches_type(object, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_schema(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.retrieve_schema()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(object, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_schema(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.retrieve_schema() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(object, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_partial(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_partial_with_all_params(self, async_client: AsyncQuark) -> None:
        role = await async_client.models.roles.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            updated_at=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_partial(self, async_client: AsyncQuark) -> None:
        response = await async_client.models.roles.with_raw_response.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        role = await response.parse()
        assert_matches_type(MlModelRole, role, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_partial(self, async_client: AsyncQuark) -> None:
        async with async_client.models.roles.with_streaming_response.update_partial(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            config_override={},
            model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            role = await response.parse()
            assert_matches_type(MlModelRole, role, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_partial(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.models.roles.with_raw_response.update_partial(
                id="",
                config_override={},
                model_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                name="name",
                owned_by_identity_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .extractor_list_params import ExtractorListParams as ExtractorListParams
from .classifier_list_params import ClassifierListParams as ClassifierListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["IdentityModel"]


class IdentityModel(BaseModel):
    disabled: bool

    email_verified: bool

    external: bool

    identity_id: str

    identity_type: Literal["User", "SystemInternal", "SystemExternal"]

    username: str

    avatar_url: Optional[str] = None

    created_at: Optional[datetime] = None

    display_name: Optional[str] = None

    email: Optional[str] = None

    external_id: Optional[str] = None

    external_updated_at: Optional[datetime] = None

    last_login: Optional[datetime] = None

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.admin.identity.identity_model import IdentityModel

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def retrieve_me(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityModel:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/users/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityModel,
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def retrieve_me(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityModel:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/users/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityModel,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve_me = to_raw_response_wrapper(
            users.retrieve_me,
        )


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve_me = async_to_raw_response_wrapper(
            users.retrieve_me,
        )


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve_me = to_streamed_response_wrapper(
            users.retrieve_me,
        )


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve_me = async_to_streamed_response_wrapper(
            users.retrieve_me,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .extractors import (
    ExtractorsResource,
    AsyncExtractorsResource,
    ExtractorsResourceWithRawResponse,
    AsyncExtractorsResourceWithRawResponse,
    ExtractorsResourceWithStreamingResponse,
    AsyncExtractorsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .classifiers import (
    ClassifiersResource,
    AsyncClassifiersResource,
    ClassifiersResourceWithRawResponse,
    AsyncClassifiersResourceWithRawResponse,
    ClassifiersResourceWithStreamingResponse,
    AsyncClassifiersResourceWithStreamingResponse,
)

__all__ = ["ContextResource", "AsyncContextResource"]


class ContextResource(SyncAPIResource):
    @cached_property
    def extractors(self) -> ExtractorsResource:
        return ExtractorsResource(self._client)

    @cached_property
    def classifiers(self) -> ClassifiersResource:
        return ClassifiersResource(self._client)

    @cached_property
    def with_raw_response(self) -> ContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return ContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return ContextResourceWithStreamingResponse(self)


class AsyncContextResource(AsyncAPIResource):
    @cached_property
    def extractors(self) -> AsyncExtractorsResource:
        return AsyncExtractorsResource(self._client)

    @cached_property
    def classifiers(self) -> AsyncClassifiersResource:
        return AsyncClassifiersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncContextResourceWithStreamingResponse(self)


class ContextResourceWithRawResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

    @cached_property
    def extractors(self) -> ExtractorsResourceWithRawResponse:
        return ExtractorsResourceWithRawResponse(self._context.extractors)

    @cached_property
    def classifiers(self) -> ClassifiersResourceWithRawResponse:
        return ClassifiersResourceWithRawResponse(self._context.classifiers)


class AsyncContextResourceWithRawResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

    @cached_property
    def extractors(self) -> AsyncExtractorsResourceWithRawResponse:
        return AsyncExtractorsResourceWithRawResponse(self._context.extractors)

    @cached_property
    def classifiers(self) -> AsyncClassifiersResourceWithRawResponse:
        return AsyncClassifiersResourceWithRawResponse(self._context.classifiers)


class ContextResourceWithStreamingResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

    @cached_property
    def extractors(self) -> ExtractorsResourceWithStreamingResponse:
        return ExtractorsResourceWithStreamingResponse(self._context.extractors)

    @cached_property
    def classifiers(self) -> ClassifiersResourceWithStreamingResponse:
        return ClassifiersResourceWithStreamingResponse(self._context.classifiers)


class AsyncContextResourceWithStreamingResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

    @cached_property
    def extractors(self) -> AsyncExtractorsResourceWithStreamingResponse:
        return AsyncExtractorsResourceWithStreamingResponse(self._context.extractors)

    @cached_property
    def classifiers(self) -> AsyncClassifiersResourceWithStreamingResponse:
        return AsyncClassifiersResourceWithStreamingResponse(self._context.classifiers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .opendal import (
    OpendalResource,
    AsyncOpendalResource,
    OpendalResourceWithRawResponse,
    AsyncOpendalResourceWithRawResponse,
    OpendalResourceWithStreamingResponse,
    AsyncOpendalResourceWithStreamingResponse,
)
from .s3_read_csv import (
    S3ReadCsvResource,
    AsyncS3ReadCsvResource,
    S3ReadCsvResourceWithRawResponse,
    AsyncS3ReadCsvResourceWithRawResponse,
    S3ReadCsvResourceWithStreamingResponse,
    AsyncS3ReadCsvResourceWithStreamingResponse,
)
from .s3_read_files_binary import (
    S3ReadFilesBinaryResource,
    AsyncS3ReadFilesBinaryResource,
    S3ReadFilesBinaryResourceWithRawResponse,
    AsyncS3ReadFilesBinaryResourceWithRawResponse,
    S3ReadFilesBinaryResourceWithStreamingResponse,
    AsyncS3ReadFilesBinaryResourceWithStreamingResponse,
)

__all__ = [
    "S3ReadFilesBinaryResource",
    "AsyncS3ReadFilesBinaryResource",
    "S3ReadFilesBinaryResourceWithRawResponse",
    "AsyncS3ReadFilesBinaryResourceWithRawResponse",
    "S3ReadFilesBinaryResourceWithStreamingResponse",
    "AsyncS3ReadFilesBinaryResourceWithStreamingResponse",
    "S3ReadCsvResource",
    "AsyncS3ReadCsvResource",
    "S3ReadCsvResourceWithRawResponse",
    "AsyncS3ReadCsvResourceWithRawResponse",
    "S3ReadCsvResourceWithStreamingResponse",
    "AsyncS3ReadCsvResourceWithStreamingResponse",
    "OpendalResource",
    "AsyncOpendalResource",
    "OpendalResourceWithRawResponse",
    "AsyncOpendalResourceWithRawResponse",
    "OpendalResourceWithStreamingResponse",
    "AsyncOpendalResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .quark_file_object_status import QuarkFileObjectStatus

__all__ = [
    "OpendalRunParams",
    "Config",
    "ConfigOpendalConfigInputOpendalS3Config",
    "ConfigOpendalConfigInputOpendalMemoryConfig",
    "ConfigOpendalConfigInputOpendalGDriveConfig",
    "ConfigOpendalConfigInputOpendalOneDriveConfig",
]


class OpendalRunParams(TypedDict, total=False):
    config: Required[Config]
    """OpenAPI compatible set of configs"""

    flow_id: Required[str]

    source_id: Required[str]

    opt_paths: List[str]
    """Filter read by paths (directories or files)"""

    opt_recursive: bool

    opt_set_status: QuarkFileObjectStatus


class ConfigOpendalConfigInputOpendalS3Config(TypedDict, total=False):
    bucket: Required[str]
    """bucket name of this backend.

    required.
    """

    type: Required[Literal["S3"]]

    access_key_id: str
    """access_key_id of this backend.

    - If access_key_id is set, we will take user's input first.
    - If not, we will try to load it from environment.
    """

    allow_anonymous: bool
    """
    Allow anonymous will allow opendal to send request without signing when
    credential is not loaded.
    """

    batch_max_operations: int
    """Set maximum batch operations of this backend.

    Some compatible services have a limit on the number of operations in a batch
    request. For example, R2 could return `Internal Error` while batch delete 1000
    files.

    Please tune this value based on services' document.
    """

    checksum_algorithm: str
    """
    Checksum Algorithm to use when sending checksums in HTTP headers. This is
    necessary when writing to AWS S3 Buckets with Object Lock enabled for example.

    Available options:

    - "crc32c"
    """

    default_storage_class: str
    """default storage_class for this backend.

    Available values:

    - `DEEP_ARCHIVE`
    - `GLACIER`
    - `GLACIER_IR`
    - `INTELLIGENT_TIERING`
    - `ONEZONE_IA`
    - `EXPRESS_ONEZONE`
    - `OUTPOSTS`
    - `REDUCED_REDUNDANCY`
    - `STANDARD`
    - `STANDARD_IA`

    S3 compatible services don't support all of them
    """

    delete_max_size: int
    """Set the maximum delete size of this backend.

    Some compatible services have a limit on the number of operations in a batch
    request. For example, R2 could return `Internal Error` while batch delete 1000
    files.

    Please tune this value based on services' document.
    """

    disable_config_load: bool
    """Disable config load so that opendal will not load config from environment.

    For examples:

    - envs like `AWS_ACCESS_KEY_ID`
    - files like `~/.aws/config`
    """

    disable_ec2_metadata: bool
    """Disable load credential from ec2 metadata.

    This option is used to disable the default behavior of opendal to load
    credential from ec2 metadata, a.k.a, IMDSv2
    """

    disable_list_objects_v2: bool
    """
    OpenDAL uses List Objects V2 by default to list objects. However, some legacy
    services do not yet support V2. This option allows users to switch back to the
    older List Objects V1.
    """

    disable_stat_with_override: bool
    """
    Disable stat with override so that opendal will not send stat request with
    override queries.

    For example, R2 doesn't support stat with `response_content_type` query.
    """

    disable_write_with_if_match: bool
    """
    Disable write with if match so that opendal will not send write request with if
    match headers.

    For example, Ceph RADOS S3 doesn't support write with if match.
    """

    enable_request_payer: bool
    """
    Indicates whether the client agrees to pay for the requests made to the S3
    bucket.
    """

    enable_versioning: bool
    """# Bucket versioning"""

    enable_virtual_host_style: bool
    """
    Enable virtual host style so that opendal will send API requests in virtual host
    style instead of path style.

    - By default, opendal will send API to
      `https://s3.us-east-1.amazonaws.com/bucket_name`
    - Enabled, opendal will send API to
      `https://bucket_name.s3.us-east-1.amazonaws.com`
    """

    enable_write_with_append: bool
    """
    Enable write with append so that opendal will send write request with append
    headers.
    """

    endpoint: str
    """endpoint of this backend.

    Endpoint must be full uri, e.g.

    - AWS S3: `https://s3.amazonaws.com` or `https://s3.{region}.amazonaws.com`
    - Cloudflare R2: `https://<ACCOUNT_ID>.r2.cloudflarestorage.com`
    - Aliyun OSS: `https://{region}.aliyuncs.com`
    - Tencent COS: `https://cos.{region}.myqcloud.com`
    - Minio: `http://127.0.0.1:9000`

    If user inputs endpoint without scheme like "s3.amazonaws.com", we will prepend
    "https://" before it.

    - If endpoint is set, we will take user's input first.
    - If not, we will try to load it from environment.
    - If still not set, default to `https://s3.amazonaws.com`.
    """

    external_id: str
    """external_id for this backend."""

    region: str
    """Region represent the signing region of this endpoint.

    This is required if you are using the default AWS S3 endpoint.

    If using a custom endpoint,

    - If region is set, we will take user's input first.
    - If not, we will try to load it from environment.
    """

    role_arn: str
    """role_arn for this backend.

    If `role_arn` is set, we will use already known config as source credential to
    assume role with `role_arn`.
    """

    role_session_name: str
    """role_session_name for this backend."""

    root: str
    """# Root Path

    All operations will happen under this root (relative
    """

    secret_access_key: str
    """secret_access_key of this backend.

    - If secret_access_key is set, we will take user's input first.
    - If not, we will try to load it from environment.
    """

    server_side_encryption: str
    """server_side_encryption for this backend.

    Available values: `AES256`, `aws:kms`.
    """

    server_side_encryption_aws_kms_key_id: str
    """server_side_encryption_aws_kms_key_id for this backend

    - If `server_side_encryption` set to `aws:kms`, and
      `server_side_encryption_aws_kms_key_id` is not set, S3 will use aws managed
      kms key to encrypt data.
    - If `server_side_encryption` set to `aws:kms`, and
      `server_side_encryption_aws_kms_key_id` is a valid kms key id, S3 will use the
      provided kms key to encrypt data.
    - If the `server_side_encryption_aws_kms_key_id` is invalid or not found, an
      error will be returned.
    - If `server_side_encryption` is not `aws:kms`, setting
      `server_side_encryption_aws_kms_key_id` is a noop.
    """

    server_side_encryption_customer_algorithm: str
    """server_side_encryption_customer_algorithm for this backend.

    Available values: `AES256`.
    """

    server_side_encryption_customer_key: str
    """server_side_encryption_customer_key for this backend.

    Value: BASE64-encoded key that matches algorithm specified in
    `server_side_encryption_customer_algorithm`.
    """

    server_side_encryption_customer_key_md5: str
    """Set server_side_encryption_customer_key_md5 for this backend.

    Value: MD5 digest of key specified in `server_side_encryption_customer_key`.
    """

    session_token: str
    """session_token (aka, security token) of this backend.

    This token will expire after sometime, it's recommended to set session_token by
    hand.
    """


class ConfigOpendalConfigInputOpendalMemoryConfig(TypedDict, total=False):
    type: Required[Literal["Mem"]]

    root: str
    """root of the backend."""


class ConfigOpendalConfigInputOpendalGDriveConfig(TypedDict, total=False):
    type: Required[Literal["GDrive"]]

    access_token: str
    """Access token for gdrive."""

    client_id: str
    """Client id for gdrive."""

    client_secret: str
    """Client secret for gdrive."""

    refresh_token: str
    """Refresh token for gdrive."""

    root: str
    """The root for gdrive"""


class ConfigOpendalConfigInputOpendalOneDriveConfig(TypedDict, total=False):
    type: Required[Literal["OneDrive"]]

    access_token: str
    """Microsoft Graph API (also OneDrive API) access token"""

    client_id: str
    """
    Microsoft Graph API Application (client) ID that is in the Azure's app
    registration portal
    """

    client_secret: str
    """
    Microsoft Graph API Application client secret that is in the Azure's app
    registration portal
    """

    enable_versioning: bool
    """Enabling version support"""

    refresh_token: str
    """Microsoft Graph API (also OneDrive API) refresh token"""

    root: str
    """The root path for the OneDrive service for the file access"""


Config: TypeAlias = Union[
    ConfigOpendalConfigInputOpendalS3Config,
    ConfigOpendalConfigInputOpendalMemoryConfig,
    ConfigOpendalConfigInputOpendalGDriveConfig,
    ConfigOpendalConfigInputOpendalOneDriveConfig,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from .snowflake_read import (
    SnowflakeReadResource,
    AsyncSnowflakeReadResource,
    SnowflakeReadResourceWithRawResponse,
    AsyncSnowflakeReadResourceWithRawResponse,
    SnowflakeReadResourceWithStreamingResponse,
    AsyncSnowflakeReadResourceWithStreamingResponse,
)

__all__ = ["DatabasesResource", "AsyncDatabasesResource"]


class DatabasesResource(SyncAPIResource):
    @cached_property
    def snowflake_read(self) -> SnowflakeReadResource:
        return SnowflakeReadResource(self._client)

    @cached_property
    def with_raw_response(self) -> DatabasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return DatabasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatabasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return DatabasesResourceWithStreamingResponse(self)


class AsyncDatabasesResource(AsyncAPIResource):
    @cached_property
    def snowflake_read(self) -> AsyncSnowflakeReadResource:
        return AsyncSnowflakeReadResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDatabasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncDatabasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatabasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncDatabasesResourceWithStreamingResponse(self)


class DatabasesResourceWithRawResponse:
    def __init__(self, databases: DatabasesResource) -> None:
        self._databases = databases

    @cached_property
    def snowflake_read(self) -> SnowflakeReadResourceWithRawResponse:
        return SnowflakeReadResourceWithRawResponse(self._databases.snowflake_read)


class AsyncDatabasesResourceWithRawResponse:
    def __init__(self, databases: AsyncDatabasesResource) -> None:
        self._databases = databases

    @cached_property
    def snowflake_read(self) -> AsyncSnowflakeReadResourceWithRawResponse:
        return AsyncSnowflakeReadResourceWithRawResponse(self._databases.snowflake_read)


class DatabasesResourceWithStreamingResponse:
    def __init__(self, databases: DatabasesResource) -> None:
        self._databases = databases

    @cached_property
    def snowflake_read(self) -> SnowflakeReadResourceWithStreamingResponse:
        return SnowflakeReadResourceWithStreamingResponse(self._databases.snowflake_read)


class AsyncDatabasesResourceWithStreamingResponse:
    def __init__(self, databases: AsyncDatabasesResource) -> None:
        self._databases = databases

    @cached_property
    def snowflake_read(self) -> AsyncSnowflakeReadResourceWithStreamingResponse:
        return AsyncSnowflakeReadResourceWithStreamingResponse(self._databases.snowflake_read)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["APIKeyListResponse", "APIKeyListResponseItem"]


class APIKeyListResponseItem(BaseModel):
    active: bool
    """# Active"""

    created_at: datetime

    identity_api_key_id: str
    """# Key ID"""

    identity_id: str
    """# Identity"""

    name: str
    """# Key Name"""

    prefix: str
    """# Key Prefix"""

    updated_at: datetime

    expires_at: Optional[datetime] = None
    """# Expiration Date"""


APIKeyListResponse: TypeAlias = List[APIKeyListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .reference_depth import ReferenceDepth

__all__ = ["Extractor"]


class Extractor(BaseModel):
    data_type: Literal["String", "Boolean", "Integer", "Float", "Rating", "Object", "Date", "Label"]
    """# Type of data to extract"""

    api_model_role_id: str = FieldInfo(alias="model_role_id")
    """# Model Role to use for extraction"""

    name: str
    """# Extractor Name"""

    owned_by_identity_id: str
    """# Owner"""

    reference_depth: ReferenceDepth
    """# Match Segments or Sentences"""

    add_reason: Optional[bool] = None
    """# Add reason for extracting this data"""

    add_references: Optional[bool] = None
    """# Add references to the extracted data

    Default is true
    """

    config: Optional[object] = None

    created_at: Optional[datetime] = None
    """# Created"""

    description: Optional[str] = None
    """# Extractor Description

    Note: the LLM uses this to perform the extraction, so be descriptive
    """

    examples: Optional[object] = None
    """# Examples of the data to extract"""

    extractor_id: Optional[str] = None
    """# Extractor ID"""

    updated_at: Optional[datetime] = None
    """# Updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.profile import api_key_update_params
from ...types.profile.api_key_list_response import APIKeyListResponse
from ...types.profile.api_key_update_response import APIKeyUpdateResponse
from ...types.context.success_response_message import SuccessResponseMessage

__all__ = ["APIKeysResource", "AsyncAPIKeysResource"]


class APIKeysResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> APIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return APIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> APIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return APIKeysResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        name: str,
        expires_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyUpdateResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/profile/api_keys",
            body=maybe_transform(
                {
                    "name": name,
                    "expires_at": expires_at,
                },
                api_key_update_params.APIKeyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/profile/api_keys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._delete(
            f"/profile/api_keys/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    def disable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._patch(
            f"/profile/api_keys/{id}/disable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/profile/api_keys/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAPIKeysResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAPIKeysResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncAPIKeysResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAPIKeysResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncAPIKeysResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        name: str,
        expires_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyUpdateResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/profile/api_keys",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "expires_at": expires_at,
                },
                api_key_update_params.APIKeyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> APIKeyListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/profile/api_keys",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIKeyListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._delete(
            f"/profile/api_keys/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    async def disable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._patch(
            f"/profile/api_keys/{id}/disable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    async def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/profile/api_keys/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class APIKeysResourceWithRawResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.update = to_raw_response_wrapper(
            api_keys.update,
        )
        self.list = to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = to_raw_response_wrapper(
            api_keys.delete,
        )
        self.disable = to_raw_response_wrapper(
            api_keys.disable,
        )
        self.retrieve_schema = to_raw_response_wrapper(
            api_keys.retrieve_schema,
        )


class AsyncAPIKeysResourceWithRawResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.update = async_to_raw_response_wrapper(
            api_keys.update,
        )
        self.list = async_to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = async_to_raw_response_wrapper(
            api_keys.delete,
        )
        self.disable = async_to_raw_response_wrapper(
            api_keys.disable,
        )
        self.retrieve_schema = async_to_raw_response_wrapper(
            api_keys.retrieve_schema,
        )


class APIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: APIKeysResource) -> None:
        self._api_keys = api_keys

        self.update = to_streamed_response_wrapper(
            api_keys.update,
        )
        self.list = to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = to_streamed_response_wrapper(
            api_keys.delete,
        )
        self.disable = to_streamed_response_wrapper(
            api_keys.disable,
        )
        self.retrieve_schema = to_streamed_response_wrapper(
            api_keys.retrieve_schema,
        )


class AsyncAPIKeysResourceWithStreamingResponse:
    def __init__(self, api_keys: AsyncAPIKeysResource) -> None:
        self._api_keys = api_keys

        self.update = async_to_streamed_response_wrapper(
            api_keys.update,
        )
        self.list = async_to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            api_keys.delete,
        )
        self.disable = async_to_streamed_response_wrapper(
            api_keys.disable,
        )
        self.retrieve_schema = async_to_streamed_response_wrapper(
            api_keys.retrieve_schema,
        )
